###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: converter.py 5564 2025-07-29 15:42:31Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.component
import zope.i18nmessageid
import zope.publisher.browser

import z3c.form.interfaces
from z3c.form.converter import BaseDataConverter

import m01.fs.schema

_ = zope.i18nmessageid.MessageFactory('p01')


@zope.component.adapter(m01.fs.schema.IFileUpload,
        z3c.form.interfaces.IWidget)
class FileUploadDataConverter(BaseDataConverter):
    """A special data converter for IFileUpload

    This converter simply returns the FileUpload which wraps the input upload
    file stream

    """

    def toWidgetValue(self, value):
        """See interfaces.IDataConverter"""
        # ignore existing data, there is no need for display them
        return u''

    def toFieldValue(self, value):
        """See interfaces.IDataConverter
        
        We should allways get a FileUpload object. Don't load the data  into 
        the memory, just return the FileUpload and process the referenced file
        later in ChunkWriter.
        
        We validate here because we like to prevent to write bad data to the
        chunks collection.

        """
        if value is None or value == '':
            # When no new file is uploaded, send a signal that we do not want
            # to do anything special.
            return z3c.form.interfaces.NOT_CHANGED

        elif isinstance(value, zope.publisher.browser.FileUpload):
            # make the following attributes available in widget
            self.widget.headers = value.headers
            # get filename and return missing_value if empty
            filename = getattr(value, 'filename', '')
            if not filename:
                return self.field.missing_value

            # strip path from filename (could happen in IE)
            cleanFileName = m01.fs.schema.getCleanFileName(filename)
            # validate filename extension
            m01.fs.schema.validateFileNameExtension(cleanFileName)
            # validate allowed formats
            m01.fs.schema.validateAllowedFormat(cleanFileName,
                self.field.allowedFormats)
            self.widget.filename = cleanFileName
            return value
        else:
            # should never happen
            raise ValueError(_(u'No FileUpload given.'))



###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: tests.py 5713 2025-11-04 16:28:58Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import unittest
import doctest

import m01.fs.testing

def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite('README.txt',
            setUp=m01.fs.testing.setUpStubMongo,
            tearDown=m01.fs.testing.tearDownStubMongo,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            ),
        ))


if __name__=='__main__':
    unittest.main(defaultTest='test_suite')



###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: layer.py 2793 2012-03-03 17:07:19Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface


class IFileUploadWidgetLayer(zope.interface.Interface):
    """Simple widget layer also usable for non IBrowserRequest.""" 



###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: browser.py 4762 2018-02-16 23:53:19Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import datetime

import zope.interface
import zope.component
import zope.datetime
import zope.publisher.browser
from zope.publisher.interfaces.http import IHTTPRequest
from zope.publisher.interfaces.http import IResult
from zope.security.proxy import removeSecurityProxy

from m01.fs import interfaces


def getFileResult(context, request, blocksize=32768):
    """Prepare a IFile result and apply header infos to response."""

    context = removeSecurityProxy(context)

    # set Content-Length
    request.response.setHeader('Content-Length', str(context.size))

    # set Content-Type
    request.response.setHeader('Content-Type', context.contentType)

    # set Last-Modified
    modified = context.uploadDate

    if modified is not None and isinstance(modified, datetime.datetime):
        lmt = zope.datetime.time(modified.isoformat())

        # return cache header if asked for and no newer data available
        header = request.getHeader('If-Modified-Since', None)
        if header is not None:
            header = header.split(';')[0]
            try:
                mod_since = long(zope.datetime.time(header))
            except:
                mod_since = None
            if mod_since is not None and lmt <= mod_since:
                request.response.setStatus(304)
                return ''

        request.response.setHeader('Last-Modified',
            zope.datetime.rfc1123_date(lmt))

    # get the file chunk reader which acts as an open file in read mode
    chunkReader = interfaces.IChunkReader(context)

    # wrap our IFileReader into the given wrapper
    wrapper = request.environment.get('wsgi.file_wrapper', None)
    if wrapper is not None:
        return wrapper(chunkReader, chunkReader.readBlockSize)

    # return our file chunk reader adapter
    return chunkReader


@zope.component.adapter(interfaces.IFileBaseSchema, IHTTPRequest)
@zope.interface.implementer(IResult)
def FileResult(context, request):
    """Provides a IResult if we directly return a IFileBaseSchema as result."""
    return getFileResult(context, request)


class FileDownload(zope.publisher.browser.BrowserPage):
    """Download view for IFileBaseSchema."""

    blocksize = 32768

    def getFile(self):
        return self.context

    def __call__(self):
        """Supports data download."""
        context = self.getFile()
        # get an IResult for IFile. We could also return the IFile here, but
        # this whould end in calling the IResult adapter for (IFile, request)
        return getFileResult(context, self.request, self.blocksize)



###############################################################################
#
# Copyright (c) 2011 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: __init__.py 5713 2025-11-04 16:28:58Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import os
import sys
import tempfile

import pymongo

import zope.schema
from zope.publisher.browser import FileUpload
from zope.testing.loggingsupport import InstalledHandler

import m01.mongo.interfaces
import m01.mongo.testing
import m01.stub.testing
from m01.mongo.fieldproperty import MongoFieldProperty

from m01.fs import interfaces
import m01.fs.item

try:
    unicode                 # p01.checker.silence
except Exception as e:
    unicode = str           # p01.checker.silence


# mongo db name used for testing
TEST_DB_NAME = 'm01_fs_testing'
TEST_COLLECTION_NAME = 'test.files'
TEST_COLLECTION_FULL_NAME = '%s.%s' % (TEST_DB_NAME, TEST_COLLECTION_NAME)


###############################################################################
#
# test helper methods
#
###############################################################################

_testClient = None

def getTestClient():
    return _testClient


def getTestDatabase():
    client = getTestClient()
    return client[TEST_DB_NAME]


def getTestCollection():
    database = getTestDatabase()
    return database[TEST_COLLECTION_NAME]


def dropTestDatabase():
    client = getTestClient()
    client.drop_database(TEST_DB_NAME)


def dropTestCollection():
    client = getTestClient()
    client[TEST_DB_NAME].drop_collection(TEST_COLLECTION_NAME)


# stub mongodb server
def setUpStubMongo(test=None):
    """Setup pymongo client as test client and setup a real empty mongodb"""
    host = 'localhost'
    port = 45017
    tz_aware = True
    sandBoxDir = os.path.join(os.path.dirname(__file__), 'sandbox')
    m01.stub.testing.startMongoServer(host, port, sandBoxDir=sandBoxDir)
    # setup pymongo.MongoClient as test client
    global _testClient
    _testClient = pymongo.MongoClient(host, port, tz_aware=tz_aware)
    logger = InstalledHandler('m01.fs')
    test.globs['logger'] = logger


def tearDownStubMongo(test=None):
    """Tear down real mongodb"""
    # stop mongodb server
    sleep = 0.5
    m01.stub.testing.stopMongoServer(sleep)
    # reset test client
    global _testClient
    _testClient = None
    logger = test.globs['logger']
    logger.clear()
    logger.uninstall()
    # clear thread local transaction cache
    m01.mongo.clearThreadLocalCache()


###############################################################################
#
# test helper
#
###############################################################################

class FakeFieldStorage(object):
    """A fake field storage"""

    def __init__(self, upload, filename, headers):
        self.file = upload
        self.filename = filename
        self.headers = headers

def getFileUpload(txt, filename=None, headers=None):
    if filename is None:
        filename = 'test.txt'
    if headers is None:
        headers = {}
    if sys.version_info[0] >= 3:
        if isinstance(txt, str):
            txt = txt.encode('utf-8')
    else:
        if isinstance(txt, unicode):
            txt = txt.encode('utf-8')
    upload = tempfile.SpooledTemporaryFile(mode='w+b')
    upload.write(txt)
    upload.seek(0)
    fieldStorage = FakeFieldStorage(upload, filename, headers)
    return FileUpload(fieldStorage)


###############################################################################
#
# Public Base Tests
#
###############################################################################

class FileItemBaseTest(m01.mongo.testing.MongoItemBaseTest):
    """FileItem base test"""

    def test_providedBy_IFile(self):
        obj = self.makeTestObject()
        self.assert_(interfaces.IFile.providedBy(obj), True)

    def test_providedBy_IFileItem(self):
        obj = self.makeTestObject()
        self.assert_(interfaces.IFileItem.providedBy(obj), True)


class FileObjectBaseTest(m01.mongo.testing.MongoObjectBaseTest):
    """FileObject base test"""

    def test_providedBy_IFile(self):
        obj = self.makeTestObject()
        self.assert_(interfaces.IFile.providedBy(obj), True)

    def test_providedBy_IFileObject(self):
        obj = self.makeTestObject()
        self.assert_(interfaces.IFileObject.providedBy(obj), True)


class ImageObjectBaseTest(m01.mongo.testing.MongoObjectBaseTest):
    """imageObject base test"""

    def test_providedBy_IImage(self):
        obj = self.makeTestObject()
        self.assert_(interfaces.IImage.providedBy(obj), True)

    def test_providedBy_IImageItem(self):
        obj = self.makeTestObject()
        self.assert_(interfaces.IImageItem.providedBy(obj), True)

    def test_providedBy_IImageObject(self):
        obj = self.makeTestObject()
        self.assert_(interfaces.IImageObject.providedBy(obj), True)


###############################################################################
#
# test components
#
###############################################################################

class TestFilesCollectionMixin(object):
    """Test files collection mixin class"""

    @property
    def collection(self):
        return getTestCollection()


class ITestSchema(zope.interface.Interface):
    """Basic test schema."""

    title = zope.schema.TextLine(
        title=u'Title',
        description=u'Title',
        default=u'',
        required=True)

    description = zope.schema.Text(
        title=u'Description',
        description=u'Description',
        default=u'',
        required=False)


class ISampleFileStorageItem(ITestSchema, interfaces.IFileStorageItem):
    """Sample storage file item interface."""

    __name__ = zope.schema.TextLine(
        title=u'Title',
        description=u'Title',
        missing_value=u'',
        default=None,
        required=True)


@zope.interface.implementer(ISampleFileStorageItem)
class SampleFileStorageItem( m01.fs.item.FileStorageItem):
    """Sample file storage item."""

    title = MongoFieldProperty(ISampleFileStorageItem['title'])
    description = MongoFieldProperty(ISampleFileStorageItem['description'])

    dumpNames = ['title', 'description']


class ISampleFileStorage(m01.mongo.interfaces.IMongoStorage):
    """Sample file storage interface."""


@zope.interface.implementer(ISampleFileStorage)
class SampleFileStorage(TestFilesCollectionMixin,
    m01.mongo.storage.MongoStorage):
    """Sample file storage."""

    def __init__(self):
        pass

    def load(self, data):
        """Load data into the right mongo item."""
        return SampleFileStorageItem(data)



###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: traverser.py 5564 2025-07-29 15:42:31Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.publisher.interfaces
import zope.publisher.interfaces.browser
import m01.fs.browser


@zope.interface.implementer(zope.publisher.interfaces.browser.IBrowserPublisher)
class FileObjectByFileNameTraverser(zope.location.location.Location):
    """Dummy context for FileObject offering the filename as traversable view.

    This traverser allows us to traverse the FileObject item by it's filename

    Note: this traverser is not registered by default. You need to register
    this traverser in your application with the correct permission for your
    own FileObject implementations.

    """

    def __init__(self, context, request):
        if context is None:
            # raise error if FielObject is None
            raise zope.publisher.interfaces.NotFound(context, u'', request)
        self.context = context
        self.request = request
        self.__parent__ = context
        self.__name__ = context.filename

    def publishTraverse(self, request, name):
        """Allows to traverse the FileObject with the real filename without 
        to register a named view.
        """
        if name == self.context.filename:
            view = m01.fs.browser.FileDownload(self.context, self.request)
            view.__name__ = name
            return view
        raise zope.publisher.interfaces.NotFound(self, name, request)


###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: chunker.py 5713 2025-11-04 16:28:58Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import struct
import datetime
import hashlib
import logging
import math
import os
from io import BytesIO

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

try:
    unicode
except Exception:
    unicode = str

import zope.interface
from zope.contenttype import guess_content_type

from m01.mongo import UTC

import m01.fs.exceptions
from m01.fs import interfaces


logger = logging.getLogger('m01.fs')

###############################################################################
#
# chunk helper methods

_SEEK_SET = os.SEEK_SET
_SEEK_CUR = os.SEEK_CUR
_SEEK_END = os.SEEK_END

BLOCK_SIZE = 16 * 1024 * 1024 # 16 MB


def extractFileName(filename):
    """Stip out path in filename (happens in IE upload)"""
    # strip out the path section even if we do not remove them
    # later, because we just need to check the filename extension.
    cleanFileName = filename.split('\\')[-1]
    cleanFileName = cleanFileName.split('/')[-1]
    dottedParts = cleanFileName.split('.')
    if len(dottedParts) <= 1:
        raise m01.fs.exceptions.MissingFileNameExtension()
    return cleanFileName


def getContentType(filename):
    """Returns the content type based on the given filename"""
    return guess_content_type(filename)[0]


# def ChunkIterator(reader):
#     while True:
#         chunk = reader.read(BLOCK_SIZE)
#         if not chunk:
#             reader.close()
#             # StopIteration is not valid on py3
#             raise StopIteration()
#         yield chunk
#     reader.close()

def ChunkIterator(reader):
    try:
        while True:
            chunk = reader.read(BLOCK_SIZE)
            if not chunk:
                # end of generator (Py3-safe replacement for StopIteration)
                return
            yield chunk
    finally:
        # always close, even on error
        reader.close()


class ChunkLogging(object):
    """Logging support"""

    def _doLog(self, msg, level=logging.DEBUG):
        logger.log(level, '%s %s %s' % (self._id, self.__class__.__name__, msg))

    def info(self, msg):
        self._doLog(msg, logging.INFO)

    def debug(self, msg):
        self._doLog(msg, logging.DEBUG)

    def error(self, msg):
        self._doLog(msg, logging.ERROR)


###############################################################################
#
# chunk writer

class ChunkWriterBase(ChunkLogging):
    """File chunk writer base class"""

    def __init__(self):
        """Setup ChunkWriterBase basics"""
        self._closed = False

    @property
    def closed(self):
        """Return close marker"""
        return self._closed

    def close(self):
        """Mark the file as closed

        A closed file cannot be written any more. Calling `close` more than
        once is allowed.
        """
        if not self._closed:
            self._closed = True

    def validate(self, fileUpload):
        """Validate file upload item"""
        if self._closed:
            raise ValueError("cannot write to a closed file")
        elif not fileUpload or not fileUpload.filename:
            # empty string or None or missing filename means no upload given
            raise ValueError("Missing file upload data")
        elif self.removed:
            raise ValueError("Can't store data for removed files")

    def setFileName(self, filename):
        """Set filename (hook for adjust the given filename)"""
        self.context.filename = unicode(filename)

    def getContentTypeAndEncoding(self, filename):
        """Returns the content type based on the given filename"""
        return guess_content_type(filename)

    def setContentTypeAndEncoding(self, contentType, encoding):
        """Set content type (hook for adjust the given type)"""
        if contentType:
            self.context.contentType = unicode(contentType)
        if encoding:
            self.context.encoding = unicode(encoding)

    def setDimension(self, data):
        """Set image dimension hook used fir IImageSchema items"""
        pass

    def write(self, data, encoding=None):
        """Write data to IFile context"""
        if self._closed:
            raise ValueError("cannot write to a closed file")
        try:
            # file-like
            read = data.read
        except AttributeError:
            # string
            if not isinstance(data, basestring):
                raise TypeError(
                    "can only write base string or file-like objects not %s" %
                        type(data))
            read = StringIO(data).read

        # write data to context
        data = read()
        # calculate and set md5
        digester = hashlib.md5()
        digester.update(data)
        self.size = len(data)
        self.setDimension(data)
        self.context.data = data
        return digester

    def addData(self, data, filename, contentType, encoding=None):
        """Add file like item or string data as chunk"""
        success = False
        try:
            # make sure we begin at the start
            digester = self.write(data)
            md5 = digester.hexdigest()
            # marker for successfully added chunk
            success = True
        except Exception as e:
            self.error('add caused an error')
            # and raise the exception
            raise e
        if success:
            # apply metadata to adapted IFile
            self.setFileName(filename)
            self.setContentTypeAndEncoding(contentType, encoding)
            self.context.md5 = unicode(md5)
            self.context.size = self.size
            self.context.uploadDate = datetime.datetime.now(UTC)
            self.debug('success')

    def add(self, fileUpload):
        """Add file upload item as chunk"""
        success = False
        self.validate(fileUpload)
        filename = extractFileName(fileUpload.filename)
        contentType, encoding = self.getContentTypeAndEncoding(filename)
        try:
            # make sure we begin at the start
            fileUpload.seek(0)
            digester = self.write(fileUpload)
            md5 = digester.hexdigest()
            # marker for successfully added chunk
            success = True
        except Exception as e:
            self.error('add caused an error')
            # and raise the exception
            raise e
        finally:
            self.close()
        if success:
            # apply metadata to adapted IFile
            self.setFileName(filename)
            self.setContentTypeAndEncoding(contentType, encoding)
            self.context.md5 = unicode(md5)
            self.context.size = self.size
            self.context.uploadDate = datetime.datetime.now(UTC)
            self.debug('success')

    def __enter__(self):
        """Support for the context manager protocol"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Support for the context manager protocol.

        Close the file and allow exceptions to propogate.
        """
        self.close()
        # propogate exceptions
        return False


###############################################################################
#
# IFile chunk reader

@zope.interface.implementer(interfaces.IFileChunkWriter)
@zope.component.adapter(interfaces.IFile)
class FileChunkWriter(ChunkWriterBase):
    """File chunk writer adapter"""

    def __init__(self, context):
        """Setup ChunkWriter based on adapted IFile context"""
        super(FileChunkWriter, self).__init__()
        self.context = context
        self._id = context._id
        if not self._id:
            raise ValueError("Missing mongo ObjectId", self._id)
        self.removed = self.context.removed


###############################################################################
#
# IImage chunk reader

def getImageDimension(data):
    data = bytes(data)
    size = len(data)
    height = -1
    width = -1

    # 'image/gif'
    if (size >= 10) and data[:6] in (b'GIF87a', b'GIF89a'):
        w, h = struct.unpack(b"<HH", data[6:10])
        width = int(w)
        height = int(h)

    # 'image/png'
    # See PNG 2. Edition spec (http://www.w3.org/TR/PNG/)
    # Bytes 0-7 are below, 4-byte chunk length, then 'IHDR'
    # and finally the 4-byte width, height
    elif ((size >= 24) and data.startswith(b'\211PNG\r\n\032\n')
          and (data[12:16] == b'IHDR')):
        w, h = struct.unpack(b">LL", data[16:24])
        width = int(w)
        height = int(h)

    # 'image/png'
    # Maybe this is for an older PNG version.
    elif (size >= 16) and data.startswith(b'\211PNG\r\n\032\n'):
        # Check to see if we have the right content type
        w, h = struct.unpack(">LL", data[8:16])
        width = int(w)
        height = int(h)

    # 'image/jpeg'
    elif (size >= 2) and data.startswith(b'\xff\xd8'):
        jpeg = BytesIO(data)
        jpeg.read(2)
        b = jpeg.read(1)
        try:
            w = -1
            h = -1
            while b and ord(b) != 0xDA:
                while ord(b) != 0xFF:
                    b = jpeg.read(1)
                while ord(b) == 0xFF:
                    b = jpeg.read(1)
                if 0xC0 <= ord(b) <= 0xC3:
                    jpeg.read(3)
                    h, w = struct.unpack(b">HH", jpeg.read(4))
                    break
                else:
                    jpeg.read(int(struct.unpack(b">H", jpeg.read(2))[0])-2)
                b = jpeg.read(1)
            width = int(w)
            height = int(h)
        except (struct.error, ValueError):
            pass

    # 'image/x-ms-bmp'
    elif (size >= 30) and data.startswith(b'BM'):
        kind = struct.unpack(b"<H", data[14:16])[0]
        if kind == 40: # Windows 3.x bitmap
            width, height = struct.unpack(b"<LL", data[18:26])

    return width, height

@zope.interface.implementer(interfaces.IImageChunkWriter)
@zope.component.adapter(interfaces.IImage)
class ImageChunkWriter(ChunkWriterBase):
    """Image chunk writer adapter"""


    def __init__(self, context):
        """Setup ChunkWriter based on adapted IImage context"""
        super(ImageChunkWriter, self).__init__()
        self.context = context
        self._id = context._id
        if not self._id:
            raise ValueError("Missing mongo ObjectId", self._id)
        self.removed = self.context.removed

    def setDimension(self, data):
        """Set image dimension hook used fir IImageSchema items"""
        width, height = getImageDimension(data)
        self.context.width = width
        self.context.height = height


###############################################################################
#
# chunk reader

@zope.interface.implementer(interfaces.IChunkReader)
class ChunkReaderBase(ChunkLogging):
    """File chunk reader adapter"""

    def __init__(self):
        self._buffer = ""
        self._position = 0
        self._closed = False

    @property
    def __name__(self):
        """simply map filename as __name__ (common in zope)"""
        return self.context.filename

    def validate(self):
        if self.removed:
            raise ValueError("Can't read data from removed files")

    def close(self):
        """Support file-like API"""
        pass

    # def read(self, size=-1):
    #     """Read at most `size` bytes from the file data

    #     If size is negative or omitted all data get returned
    #     """
    #     # validate read operation
    #     self.validate()

    #     res = ""

    #     if size == 0:
    #         res = ""
    #     elif size < 0:
    #         # set position at the end of our data
    #         self._position = int(self.size)
    #         # return all data
    #         res = self.context.data
    #     elif self._position == 0 and size > int(self.size):
    #         # set position at the end of our data
    #         self._position = int(self.size)
    #         # return all data
    #         res = self.context.data
    #     else:
    #         # get current position
    #         position = self._position
    #         # calculate pending data size
    #         pending = int(self.size) - position
    #         if size < pending:
    #             # return oly partial data based on position and size
    #             self._position = position + size
    #             res = self.context.data[position:end]
    #         else:
    #             # set position at the end out our data
    #             self._position = position + pending
    #             # return less then asked for because thre is not more data
    #             res = self.context.data[position:]

    #     # return a string and not the Binary instance
    #     return str(res)

    def read(self, size=-1):
        """Read at most `size` bytes from MongoBinary."""
        self.validate()

        # always binary
        data = self.context.data
        data_size = int(self.size)
        pos = self._position

        # empty read
        if size == 0:
            # always bytes
            return b""

        # full read
        if size < 0 or (pos == 0 and size >= data_size):
            self._position = data_size
            return data

        # partial read
        pending = data_size - pos
        if size < pending:
            self._position = pos + size
            return data[pos:pos + size]

        # last chunk
        self._position = data_size
        return data[pos:]

    def readline(self, size=-1):
        """Read one line or up to `size` bytes from the file data"""
        bytes = ""
        while len(bytes) != size:
            byte = self.read(1)
            bytes += byte
            if byte == "" or byte == "\n":
                break
        return bytes

    def tell(self):
        """Return the current position of this file"""
        return self._position

    def seek(self, pos, whence=_SEEK_SET):
        """Set the current position of this file"""
        if whence == _SEEK_SET:
            new_pos = pos
        elif whence == _SEEK_CUR:
            new_pos = self._position + pos
        elif whence == _SEEK_END:
            new_pos = int(self.size) + pos
        else:
            raise IOError(22, "Invalid value for `whence`")
        if new_pos < 0:
            raise IOError(22, "Invalid value for `pos` - must be positive")
        self._position = new_pos
        self._buffer = ""

    def __iter__(self):
        """Return an iterator over all of this file's data"""
        # validate read operation
        self.validate()
        # return iterator
        return ChunkIterator(self)

    def __enter__(self):
        """Makes it possible to use with the context manager protocol"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Makes it possible to use with the context manager protocol"""
        return False


###############################################################################
#
# IFile chunk reader

@zope.component.adapter(interfaces.IFile)
class FileChunkReader(ChunkReaderBase):
    """File chunk reader adapter"""

    def __init__(self, context):
        """Setup chunk reader based on adapted context"""
        super(FileChunkReader, self).__init__()
        self.context = context
        self._id = context._id
        if not self._id:
            raise ValueError("Missing mongo ObjectId", self._id)
        self.size = self.context.size
        self.md5 = self.context.md5
        self.filename = self.context.filename
        self.contentType = self.context.contentType
        self.uploadDate = self.context.uploadDate
        self.removed = self.context.removed


class FileChunkDataReader(ChunkReaderBase):
    """Chunk data reader based on given mongodb file data"""

    def __init__(self, data):
        """Setup chunk reader based on given file data"""
        super(FileChunkDataReader, self).__init__()
        self._id = data['_id']
        if not self._id:
            raise ValueError("Missing mongo ObjectId", self._id)
        self.size = data['size']
        self.md5 = data['md5']
        self.filename = data['filename']
        self.contentType = data['contentType']
        self.uploadDate = data['uploadDate']
        self.removed = data['removed']


def getFileChunkDataReader(collection, query):
    """Lookup a file and return a FileChunkDataReader"""
    # get data
    data = collection.find_one(query)
    return FileChunkDataReader(data)


###############################################################################
#
# IImage chunk reader

@zope.component.adapter(interfaces.IImage)
class ImageChunkReader(ChunkReaderBase):
    """Image chunk reader adapter"""

    def __init__(self, context):
        """Setup chunk reader based on adapted context"""
        super(ImageChunkReader, self).__init__()
        self.context = context
        self._id = context._id
        if not self._id:
            raise ValueError("Missing mongo ObjectId", self._id)
        self.size = self.context.size
        self.md5 = self.context.md5
        self.filename = self.context.filename
        self.contentType = self.context.contentType
        self.uploadDate = self.context.uploadDate
        self.removed = self.context.removed


class ImageChunkDataReader(ChunkReaderBase):
    """Chunk data reader based on given mongodb file data"""

    def __init__(self, data):
        """Setup chunk reader based on given file data"""
        super(ImageChunkDataReader, self).__init__()
        self._id = data['_id']
        if not self._id:
            raise ValueError("Missing mongo ObjectId", self._id)
        self.size = data['size']
        self.md5 = data['md5']
        self.filename = data['filename']
        self.contentType = data['contentType']
        self.uploadDate = data['uploadDate']
        self.removed = data['removed']

def getImageChunkDataReader(collection, query):
    """Lookup a file and return a ImageChunkDataReader"""
    # get data
    data = collection.find_one(query)
    return ImageChunkDataReader(data)



###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: interfaces.py 4784 2018-03-01 01:28:55Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import os
import zope.schema
import zope.i18nmessageid
from zope.publisher.interfaces.http import IResult

import z3c.form.interfaces

import m01.mongo.interfaces
import m01.mongo.schema

import m01.fs.schema

_ = zope.i18nmessageid.MessageFactory('p01')


_SEEK_SET = os.SEEK_SET


###############################################################################
#
# widget

class IFileUploadWidget(z3c.form.interfaces.IFileWidget):
    """Widget for IFileUpload field.

    The FileUploadDataConverter for this widget returns the plain
    FileUpload which wraps the tmp file.

    see: zope.publisher.browser.FileUpload
    """


# upload widget schema (ignoreContext=True)
class IFileUploadSchema(zope.interface.Interface):
    """Schema for render a FileUpload widget"""

    # used in z3c.form for render a IFileUploadWidget with ignoreContext=True
    fileUpload = m01.fs.schema.FileUpload(
        title=_(u'File Upload'),
        description=_(u'File Upload'),
        default=None,
        required=True)


###############################################################################
#
# IFile base schema shared with IFile and IImage schema

class IFileBaseSchema(zope.interface.Interface):
    """File base schema without method definition.

    Note: This interface is used for IFile, IImage and IChunkReader
    """

    data = m01.mongo.schema.MongoBinary(
        title=u'File data',
        description=u'File data',
        required=True)

    filename = zope.schema.TextLine(
        title=_(u'Filename'),
        description=_(u'Filename'),
        required=True)

    contentType = zope.schema.TextLine(
        title=_(u'Content Type'),
        description=_(u'Content Type'),
        required=True)

    encoding = zope.schema.TextLine(
        title=_(u'Encoding'),
        description=_(u'Encoding'),
        required=False)

    size = zope.schema.Int(
        title=_(u'File size'),
        description=_(u'File size'),
        default=0,
        required=True)

    md5 = zope.schema.TextLine(
        title=_(u'MD5'),
        description=_(u'MD5'),
        default=None,
        required=True)

    uploadDate = zope.schema.Datetime(
        title=_(u'Upload date'),
        description=_(u'Upload date'),
        default=None,
        required=True)

    removed = zope.schema.Bool(
        title=u'removed marker',
        description=u'removed marker',
        default=False,
        required=True)


###############################################################################
#
# IFile

class IFileSchema(IFileBaseSchema):
    """File schema without method definition.

    Note: This interface is used for IFile and IChunkReader
    """


class IFile(IFileSchema):
    """File without ILocation.

    Note: This interface doens't provide ILocation or IContained. The missing
    ILocation interface in this interface makes it simpler for define correct
    permissions in ZCML.
    """

    def getFileWriter():
        """Returns a IChunkReader"""

    def getFileReader():
        """Returns a IChunkReader"""

    def applyFileUpload(fileUpload):
        """Apply FileUpload given from request publisher"""


# IFileItem

class IFileItem(IFile, m01.mongo.interfaces.IMongoItem):
    """Mongo file providing IMongoItem"""


class IFileStorageItem(IFileItem, m01.mongo.interfaces.IMongoStorageItem):
    """IMongoStorageItem providing IFile."""


class ISecureFileStorageItem(IFileItem,
    m01.mongo.interfaces.ISecureMongoStorageItem):
    """ISecureMongoStorageItem providing IFile."""


class IFileContainerItem(IFileItem, m01.mongo.interfaces.IMongoContainerItem):
    """IMongoContainerItem providing IFile."""


class ISecureFileContainerItem(IFileItem,
    m01.mongo.interfaces.ISecureMongoContainerItem):
    """ISecureMongoContainerItem providing IFile."""


###############################################################################
#
# IFileObject

class IFileObject(IFileItem, m01.mongo.interfaces.IMongoObject):
    """IMongoObject providing IFile."""


###############################################################################
#
# IImage

class IImageSchema(IFileBaseSchema):
    """Image schema"""

    width = zope.schema.Int(
        title=_(u'Image width'),
        description=_(u'Image width'),
        default=0,
        required=False)

    height = zope.schema.Int(
        title=_(u'Image height'),
        description=_(u'Image height'),
        default=0,
        required=False)


class IImage(IImageSchema):
    """Image without ILocation.

    Note: This interface doens't provide ILocation or IContained. The missing
    ILocation interface in this interface makes it simpler for define correct
    permissions in ZCML.
    """

    def getFileWriter():
        """Returns a IChunkReader"""

    def getFileReader():
        """Returns a IChunkReader"""

    def applyFileUpload(fileUpload):
        """Apply FileUpload given from request publisher"""


class IImageItem(IImage, m01.mongo.interfaces.IMongoItem):
    """Mongo file providing IMongoItem"""


class IImageStorageItem(IImageItem, m01.mongo.interfaces.IMongoStorageItem):
    """IMongoStorageItem providing IImage."""


class ISecureImageStorageItem(IImageItem,
    m01.mongo.interfaces.ISecureMongoStorageItem):
    """ISecureMongoStorageItem providing IImage."""


class IImageContainerItem(IImageItem, m01.mongo.interfaces.IMongoContainerItem):
    """IMongoContainerItem providing IImage."""


class ISecureImageContainerItem(IImageItem,
    m01.mongo.interfaces.ISecureMongoContainerItem):
    """ISecureMongoContainerItem providing IImage."""


###############################################################################
#
# ImageObject

class IImageObject(IImageItem, m01.mongo.interfaces.IMongoObject):
    """IMongoObject providing IImage."""


###############################################################################
#
# chunk helper

class IChunkIterator(zope.interface.Interface):
    """Chunk iterator"""

    def __iter__(self):
        """iter"""

    def next(self):
        """next"""


###############################################################################
#
# chunk writer

class IChunkWriter(zope.interface.Interface):
    """Grid data writer for IFile"""

    closed = zope.schema.Bool(
        title=u'close marker',
        description=u'close marker',
        default=False,
        readonly=True,
        required=True)

    def close():
        """Close an open file handle."""

    def write(data):
        """Write data to mongodb"""

    def add(fileUpload):
        """Add file upload data as chunk"""

    def remove():
        """Mark chunk data as removed"""

    def __enter__():
        """Support for the context manager protocol"""

    def __exit__(exc_type, exc_val, exc_tb):
        """Support for the context manager protocol.

        Close the file and allow exceptions to propogate.
        """


class IFileChunkWriter(IChunkWriter):
    """File chunk writer"""


class IImageChunkWriter(IChunkWriter):
    """Image chunk writer"""


###############################################################################
#
# chunk reader

class IChunkReader(IResult):
    """Chunk data reader and IResult schema"""

    def read(size=-1):
        """Read at most `size` bytes from the file

        If size is negative or omitted all data is read
        """

    def readline(size=-1):
        """Read one line or up to `size` bytes from the file"""

    def tell():
        """Return the current position of this file"""

    def seek(pos, whence=_SEEK_SET):
        """Set the current position of this file"""

    def __iter__():
        """Return an iterator over all of this file's data"""

    def close():
        """Support file-like API"""

    def __enter__():
        """Makes it possible to use with the context manager protocol"""

    def __exit__(exc_type, exc_val, exc_tb):
        """Makes it possible to use with the context manager protocol"""


class IFileChunkReader(IChunkReader, IFileSchema):
    """Chunk data reader for IFile and IImage also providing IResult

    A ChunkReader also provides IFileSchema attribute access to the adapted
    context

    """


class IImageChunkReader(IChunkReader, IImageSchema):
    """Chunk data reader for Image also providing IResult

    A ChunkReader also provides IImageSchema attribute access to the adapted
    context

    """



###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: item.py 5670 2025-10-08 14:44:40Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

try:
    unicode
except Exception:
    unicode = str

import zope.interface
import zope.location.interfaces

import m01.mongo.item
import m01.mongo.interfaces
from m01.mongo import LOCAL
from m01.mongo.fieldproperty import MongoFieldProperty

import m01.fs.base
from m01.fs import interfaces


###############################################################################
#
# file

@zope.interface.implementer(interfaces.IFileStorageItem,
        m01.mongo.interfaces.IMongoParentAware,
        zope.location.interfaces.ILocation)
class FileStorageItem(m01.fs.base.FileItemBase):
    """Simple mongo file item.

    This FileStorageItem will use the mongo ObjectId as the __name__. This
    means you can't set an own __name__ value for this object.

    Implement your own IFileStorageItem with the attributes you need and the
    relevant chunks collection.
    """

    _skipNames = ['__name__']

    @property
    def __name__(self):
        return unicode(self._id)


@zope.interface.implementer(interfaces.ISecureFileStorageItem,
        m01.mongo.interfaces.IMongoParentAware,
        zope.location.interfaces.ILocation)
class SecureFileStorageItem(m01.fs.base.SecureFileItemBase):
    """Security aware IFileStorageItem."""

    _skipNames = ['__name__']

    @property
    def __name__(self):
        return unicode(self._id)


@zope.interface.implementer(interfaces.IFileContainerItem,
        m01.mongo.interfaces.IMongoParentAware,
        zope.location.interfaces.ILocation)
class FileContainerItem(m01.fs.base.FileItemBase):
    """File container item.

    Implement your own IFileContainerItem with the attributes you need and the
    relevant chunks collection.
    """

    _skipNames = []

    # validate __name__ and observe to set _m_changed
    __name__ = MongoFieldProperty(
        zope.location.interfaces.ILocation['__name__'])


@zope.interface.implementer(interfaces.ISecureFileContainerItem,
        m01.mongo.interfaces.IMongoParentAware,
        zope.location.interfaces.ILocation)
class SecureFileContainerItem(m01.fs.base.SecureFileItemBase):
    """Security aware IFileStorageItem."""

    # validate __name__ and observe to set _m_changed
    __name__ = MongoFieldProperty(
        zope.location.interfaces.ILocation['__name__'])


@zope.interface.implementer(interfaces.IFileObject)
class FileObject(m01.fs.base.FileBase, m01.mongo.item.MongoObject):
    """MongoObject based file"""

    _dumpNames = ['_id', '_oid', '__name__', '_type', '_version',
                  '_field',
                  'created', 'modified', 'removed',
                  'data', 'size', 'md5', 'filename', 'contentType', 'encoding',
                  'uploadDate']

    @property
    def collection(self):
        return self.getCollection(self.__parent__)


###############################################################################
#
# image

@zope.interface.implementer(interfaces.IImageStorageItem,
        m01.mongo.interfaces.IMongoParentAware,
        zope.location.interfaces.ILocation)
class ImageStorageItem(m01.fs.base.ImageItemBase):
    """Simple mongo file item.

    This ImageStorageItem will use the mongo ObjectId as the __name__. This
    means you can't set an own __name__ value for this object.

    Implement your own IImageStorageItem with the attributes you need and the
    relevant chunks collection.
    """
    _skipNames = ['__name__']

    @property
    def __name__(self):
        return unicode(self._id)


@zope.interface.implementer(interfaces.ISecureImageStorageItem,
        m01.mongo.interfaces.IMongoParentAware,
        zope.location.interfaces.ILocation)
class SecureImageStorageItem(m01.fs.base.SecureImageItemBase):
    """Security aware IImageStorageItem."""

    _skipNames = ['__name__']

    @property
    def __name__(self):
        return unicode(self._id)


@zope.interface.implementer(interfaces.IImageContainerItem,
        m01.mongo.interfaces.IMongoParentAware,
        zope.location.interfaces.ILocation)
class ImageContainerItem(m01.fs.base.ImageItemBase):
    """Image container item.

    Implement your own IImageContainerItem with the attributes you need and the
    relevant chunks collection.
    """

    _skipNames = []

    # validate __name__ and observe to set _m_changed
    __name__ = MongoFieldProperty(
        zope.location.interfaces.ILocation['__name__'])


@zope.interface.implementer(interfaces.ISecureImageContainerItem,
        m01.mongo.interfaces.IMongoParentAware,
        zope.location.interfaces.ILocation)
class SecureImageContainerItem(m01.fs.base.SecureImageItemBase):
    """Security aware IImageStorageItem."""

    # validate __name__ and observe to set _m_changed
    __name__ = MongoFieldProperty(
        zope.location.interfaces.ILocation['__name__'])


@zope.interface.implementer(interfaces.IImageObject)
class ImageObject(m01.fs.base.ImageBase, m01.mongo.item.MongoObject):
    """MongoObject based file"""

    _dumpNames = ['_id', '_oid', '__name__', '_type', '_version',
                  '_field',
                  'created', 'modified', 'removed',
                  'data', 'size', 'md5', 'filename', 'contentType', 'encoding',
                  'uploadDate', 'width', 'height',]

    @property
    def collection(self):
        return self.getCollection(self.__parent__)



###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: __init__.py 5564 2025-07-29 15:42:31Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.component

from z3c.form import widget
import z3c.form.browser.file
from z3c.form.interfaces import IFieldWidget

import m01.fs.schema
import m01.fs.layer
from m01.fs import interfaces


# z3c.from
@zope.interface.implementer_only(interfaces.IFileUploadWidget)
class FileUploadWidget(z3c.form.browser.file.FileWidget):
    """Widget for IFileUpload field.

    The registered FileUploadDataConverter for this widget returns an
    FileUpload item
    """

    css = u'fsFileUploadWidget'


@zope.component.adapter(m01.fs.schema.IFileUpload,
                        m01.fs.layer.IFileUploadWidgetLayer)
@zope.interface.implementer(IFieldWidget)
def FileUploadFieldWidget(field, request):
    """IFieldWidget factory for FileUploadWidget."""
    return widget.FieldWidget(field, FileUploadWidget(request))



###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: schema.py 5564 2025-07-29 15:42:31Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import os

import zope.interface
import zope.schema
import zope.schema.interfaces
import zope.i18nmessageid

import m01.fs.exceptions

_ = zope.i18nmessageid.MessageFactory('p01')


def getCleanFileName(filename):
    """Strip path from filenme (could happen in IE)"""
    cleanFileName = filename.split('\\')[-1]
    return cleanFileName.split('/')[-1]

def validateFileNameExtension(fileName):
    """Check missing file name extension"""
    # strip out the path section even if we do not remove them
    # later, because we just need to check the filename extension.
    dottedParts = fileName.split('.')
    if len(dottedParts) <= 1:
        raise m01.fs.exceptions.MissingFileNameExtension()

def validateAllowedFormat(fileName, allowedFormats):
    if not '*' in allowedFormats:
        postfix = fileName.split('.')[-1].lower()
        if postfix not in allowedFormats:
            v = _('Files ending with "$postfix" are not allowed.',
                  mapping={'postfix': postfix})
            raise m01.fs.exceptions.AllowedFormatError(fileName)

def validateFileSize(f, fileName, minSize, maxSize):
    fSize = os.fstat(f.fileno()).st_size
    if minSize and minSize > fSize:
        raise m01.fs.exceptions.TooSmallFile(fileName)
    if maxSize and maxSize < fSize :
        raise m01.fs.exceptions.TooBigFile(fileName)


# IFile upload schema field
class IFileUpload(zope.schema.interfaces.IField):
    """File upload field interface."""

    minSize = zope.schema.Int(
        title=_(u'Minimum file size'),
        description=_(u'Minimum file size'),
        min=0,
        required=False,
        default=None)

    maxSize = zope.schema.Int(
        title=_(u'Maximum file size'),
        description=_(u'Maximum file size'),
        min=0,
        required=False,
        default=None)

    allowedFormats = zope.schema.Tuple(
        title=_(u'Allowed Formats'),
        description=_(u'Allowed Formats'),
        default=(u'*',),
        required=True)


@zope.interface.implementer(IFileUpload)
class FileUpload(zope.schema.Field):
    """File upload field implementation."""

    def __init__(self, allowedFormats='*', minSize=0, maxSize=None, **kw):
        self.allowedFormats = allowedFormats
        self.minSize = minSize
        self.maxSize = maxSize
        super(FileUpload, self).__init__(**kw)

    def validateFileNameExtension(self, fileName):
        """Validate allowed format"""
        validateFileNameExtension(fileName)

    def validateAllowedFormat(self, fileName, allowedFormats):
        """Validate allowed format"""
        validateAllowedFormat(fileName, allowedFormats)

    def validateFileSize(self, f, fileName, minSize, maxSize):
        validateFileSize(f, fileName, minSize, maxSize)

    def _validate(self, value):
        super(FileUpload, self)._validate(value)

        # strip out the path section
        fileName = getCleanFileName(value.filename)

        # validate filename extension
        self.validateFileNameExtension(fileName)

        # validate allowed extension
        self.validateAllowedFormat(fileName, self.allowedFormats)

        # validate file size
        self.validateFileSize(value, fileName, self.minSize, self.maxSize)



registry = dict(version=0)
def bind():
    from cPickle import loads as _loads
    _lookup_attr = _loads('cchameleon.core.codegen\nlookup_attr\np1\n.')
    _init_scope = _loads('cchameleon.core.utils\necontext\np1\n.')
    _re_amp = _loads("cre\n_compile\np1\n(S'&(?!([A-Za-z]+|#[0-9]+);)'\np2\nI0\ntRp3\n.")
    _attrs_95379088 = _loads('(dp1\nVaccesskey\np2\nV\nsVdisabled\np3\nV\nsValt\np4\nV\nsVid\np5\nV\nsVsize\np6\nV\nsVname\np7\nV\nsVtitle\np8\nV\nsVclass\np9\nV\nsVreadonly\np10\nV\nsVmaxlength\np11\nV\nsVtype\np12\nVfile\np13\nsVtabindex\np14\nV\ns.')
    _init_stream = _loads('cchameleon.core.generation\ninitialize_stream\np1\n.')
    _init_default = _loads('cchameleon.core.generation\ninitialize_default\np1\n.')
    _init_tal = _loads('cchameleon.core.generation\ninitialize_tal\np1\n.')
    _path = _loads("ccopy_reg\n_reconstructor\np1\n(cz3c.pt.expressions\nZopeTraverser\np2\nc__builtin__\nobject\np3\nNtRp4\n(dp5\nS'proxify'\np6\ncz3c.pt.expressions\nidentity\np7\nsb.")
    def render(econtext, rcontext=None):
        macros = econtext.get('macros')
        _translate = econtext.get('_translate')
        _slots = econtext.get('_slots')
        target_language = econtext.get('target_language')
        u'_init_stream()'
        (_out, _write, ) = _init_stream()
        u'_init_tal()'
        (_attributes, repeat, ) = _init_tal()
        u'_init_default()'
        _default = _init_default()
        u'None'
        default = None
        u'None'
        _domain = None
        attrs = _attrs_95379088
        u'view/id'
        _write(u'<input type="file"')
        _tmp1 = _path(econtext['view'], econtext['request'], True, 'id')
        if (_tmp1 is _default):
            _tmp1 = u''
        if ((_tmp1 is not None) and (_tmp1 is not False)):
            if (_tmp1.__class__ not in (str, unicode, int, float, )):
                _tmp1 = unicode(_translate(_tmp1, domain=_domain, mapping=None, target_language=target_language, default=None))
            else:
                if not isinstance(_tmp1, unicode):
                    _tmp1 = str(_tmp1)
            if ('&' in _tmp1):
                if (';' in _tmp1):
                    _tmp1 = _re_amp.sub('&amp;', _tmp1)
                else:
                    _tmp1 = _tmp1.replace('&', '&amp;')
            if ('<' in _tmp1):
                _tmp1 = _tmp1.replace('<', '&lt;')
            if ('>' in _tmp1):
                _tmp1 = _tmp1.replace('>', '&gt;')
            if ('"' in _tmp1):
                _tmp1 = _tmp1.replace('"', '&quot;')
            _write(((' id="' + _tmp1) + '"'))
        u'view/name'
        _tmp1 = _path(econtext['view'], econtext['request'], True, 'name')
        if (_tmp1 is _default):
            _tmp1 = u''
        if ((_tmp1 is not None) and (_tmp1 is not False)):
            if (_tmp1.__class__ not in (str, unicode, int, float, )):
                _tmp1 = unicode(_translate(_tmp1, domain=_domain, mapping=None, target_language=target_language, default=None))
            else:
                if not isinstance(_tmp1, unicode):
                    _tmp1 = str(_tmp1)
            if ('&' in _tmp1):
                if (';' in _tmp1):
                    _tmp1 = _re_amp.sub('&amp;', _tmp1)
                else:
                    _tmp1 = _tmp1.replace('&', '&amp;')
            if ('<' in _tmp1):
                _tmp1 = _tmp1.replace('<', '&lt;')
            if ('>' in _tmp1):
                _tmp1 = _tmp1.replace('>', '&gt;')
            if ('"' in _tmp1):
                _tmp1 = _tmp1.replace('"', '&quot;')
            _write(((' name="' + _tmp1) + '"'))
        u'view/css'
        _tmp1 = _path(econtext['view'], econtext['request'], True, 'css')
        if (_tmp1 is _default):
            _tmp1 = u''
        if ((_tmp1 is not None) and (_tmp1 is not False)):
            if (_tmp1.__class__ not in (str, unicode, int, float, )):
                _tmp1 = unicode(_translate(_tmp1, domain=_domain, mapping=None, target_language=target_language, default=None))
            else:
                if not isinstance(_tmp1, unicode):
                    _tmp1 = str(_tmp1)
            if ('&' in _tmp1):
                if (';' in _tmp1):
                    _tmp1 = _re_amp.sub('&amp;', _tmp1)
                else:
                    _tmp1 = _tmp1.replace('&', '&amp;')
            if ('<' in _tmp1):
                _tmp1 = _tmp1.replace('<', '&lt;')
            if ('>' in _tmp1):
                _tmp1 = _tmp1.replace('>', '&gt;')
            if ('"' in _tmp1):
                _tmp1 = _tmp1.replace('"', '&quot;')
            _write(((' class="' + _tmp1) + '"'))
        u'view/size'
        _tmp1 = _path(econtext['view'], econtext['request'], True, 'size')
        if (_tmp1 is _default):
            _tmp1 = u''
        if ((_tmp1 is not None) and (_tmp1 is not False)):
            if (_tmp1.__class__ not in (str, unicode, int, float, )):
                _tmp1 = unicode(_translate(_tmp1, domain=_domain, mapping=None, target_language=target_language, default=None))
            else:
                if not isinstance(_tmp1, unicode):
                    _tmp1 = str(_tmp1)
            if ('&' in _tmp1):
                if (';' in _tmp1):
                    _tmp1 = _re_amp.sub('&amp;', _tmp1)
                else:
                    _tmp1 = _tmp1.replace('&', '&amp;')
            if ('<' in _tmp1):
                _tmp1 = _tmp1.replace('<', '&lt;')
            if ('>' in _tmp1):
                _tmp1 = _tmp1.replace('>', '&gt;')
            if ('"' in _tmp1):
                _tmp1 = _tmp1.replace('"', '&quot;')
            _write(((' size="' + _tmp1) + '"'))
        u'view/alt'
        _tmp1 = _path(econtext['view'], econtext['request'], True, 'alt')
        if (_tmp1 is _default):
            _tmp1 = u''
        if ((_tmp1 is not None) and (_tmp1 is not False)):
            if (_tmp1.__class__ not in (str, unicode, int, float, )):
                _tmp1 = unicode(_translate(_tmp1, domain=_domain, mapping=None, target_language=target_language, default=None))
            else:
                if not isinstance(_tmp1, unicode):
                    _tmp1 = str(_tmp1)
            if ('&' in _tmp1):
                if (';' in _tmp1):
                    _tmp1 = _re_amp.sub('&amp;', _tmp1)
                else:
                    _tmp1 = _tmp1.replace('&', '&amp;')
            if ('<' in _tmp1):
                _tmp1 = _tmp1.replace('<', '&lt;')
            if ('>' in _tmp1):
                _tmp1 = _tmp1.replace('>', '&gt;')
            if ('"' in _tmp1):
                _tmp1 = _tmp1.replace('"', '&quot;')
            _write(((' alt="' + _tmp1) + '"'))
        u'view/title'
        _tmp1 = _path(econtext['view'], econtext['request'], True, 'title')
        if (_tmp1 is _default):
            _tmp1 = u''
        if ((_tmp1 is not None) and (_tmp1 is not False)):
            if (_tmp1.__class__ not in (str, unicode, int, float, )):
                _tmp1 = unicode(_translate(_tmp1, domain=_domain, mapping=None, target_language=target_language, default=None))
            else:
                if not isinstance(_tmp1, unicode):
                    _tmp1 = str(_tmp1)
            if ('&' in _tmp1):
                if (';' in _tmp1):
                    _tmp1 = _re_amp.sub('&amp;', _tmp1)
                else:
                    _tmp1 = _tmp1.replace('&', '&amp;')
            if ('<' in _tmp1):
                _tmp1 = _tmp1.replace('<', '&lt;')
            if ('>' in _tmp1):
                _tmp1 = _tmp1.replace('>', '&gt;')
            if ('"' in _tmp1):
                _tmp1 = _tmp1.replace('"', '&quot;')
            _write(((' title="' + _tmp1) + '"'))
        u'view/tabindex'
        _tmp1 = _path(econtext['view'], econtext['request'], True, 'tabindex')
        if (_tmp1 is _default):
            _tmp1 = u''
        if ((_tmp1 is not None) and (_tmp1 is not False)):
            if (_tmp1.__class__ not in (str, unicode, int, float, )):
                _tmp1 = unicode(_translate(_tmp1, domain=_domain, mapping=None, target_language=target_language, default=None))
            else:
                if not isinstance(_tmp1, unicode):
                    _tmp1 = str(_tmp1)
            if ('&' in _tmp1):
                if (';' in _tmp1):
                    _tmp1 = _re_amp.sub('&amp;', _tmp1)
                else:
                    _tmp1 = _tmp1.replace('&', '&amp;')
            if ('<' in _tmp1):
                _tmp1 = _tmp1.replace('<', '&lt;')
            if ('>' in _tmp1):
                _tmp1 = _tmp1.replace('>', '&gt;')
            if ('"' in _tmp1):
                _tmp1 = _tmp1.replace('"', '&quot;')
            _write(((' tabindex="' + _tmp1) + '"'))
        u'view/disabled'
        _tmp1 = _path(econtext['view'], econtext['request'], True, 'disabled')
        if (_tmp1 is _default):
            _tmp1 = u''
        if ((_tmp1 is not None) and (_tmp1 is not False)):
            if (_tmp1.__class__ not in (str, unicode, int, float, )):
                _tmp1 = unicode(_translate(_tmp1, domain=_domain, mapping=None, target_language=target_language, default=None))
            else:
                if not isinstance(_tmp1, unicode):
                    _tmp1 = str(_tmp1)
            if ('&' in _tmp1):
                if (';' in _tmp1):
                    _tmp1 = _re_amp.sub('&amp;', _tmp1)
                else:
                    _tmp1 = _tmp1.replace('&', '&amp;')
            if ('<' in _tmp1):
                _tmp1 = _tmp1.replace('<', '&lt;')
            if ('>' in _tmp1):
                _tmp1 = _tmp1.replace('>', '&gt;')
            if ('"' in _tmp1):
                _tmp1 = _tmp1.replace('"', '&quot;')
            _write(((' disabled="' + _tmp1) + '"'))
        u'view/readonly'
        _tmp1 = _path(econtext['view'], econtext['request'], True, 'readonly')
        if (_tmp1 is _default):
            _tmp1 = u''
        if ((_tmp1 is not None) and (_tmp1 is not False)):
            if (_tmp1.__class__ not in (str, unicode, int, float, )):
                _tmp1 = unicode(_translate(_tmp1, domain=_domain, mapping=None, target_language=target_language, default=None))
            else:
                if not isinstance(_tmp1, unicode):
                    _tmp1 = str(_tmp1)
            if ('&' in _tmp1):
                if (';' in _tmp1):
                    _tmp1 = _re_amp.sub('&amp;', _tmp1)
                else:
                    _tmp1 = _tmp1.replace('&', '&amp;')
            if ('<' in _tmp1):
                _tmp1 = _tmp1.replace('<', '&lt;')
            if ('>' in _tmp1):
                _tmp1 = _tmp1.replace('>', '&gt;')
            if ('"' in _tmp1):
                _tmp1 = _tmp1.replace('"', '&quot;')
            _write(((' readonly="' + _tmp1) + '"'))
        u'view/accesskey'
        _tmp1 = _path(econtext['view'], econtext['request'], True, 'accesskey')
        if (_tmp1 is _default):
            _tmp1 = u''
        if ((_tmp1 is not None) and (_tmp1 is not False)):
            if (_tmp1.__class__ not in (str, unicode, int, float, )):
                _tmp1 = unicode(_translate(_tmp1, domain=_domain, mapping=None, target_language=target_language, default=None))
            else:
                if not isinstance(_tmp1, unicode):
                    _tmp1 = str(_tmp1)
            if ('&' in _tmp1):
                if (';' in _tmp1):
                    _tmp1 = _re_amp.sub('&amp;', _tmp1)
                else:
                    _tmp1 = _tmp1.replace('&', '&amp;')
            if ('<' in _tmp1):
                _tmp1 = _tmp1.replace('<', '&lt;')
            if ('>' in _tmp1):
                _tmp1 = _tmp1.replace('>', '&gt;')
            if ('"' in _tmp1):
                _tmp1 = _tmp1.replace('"', '&quot;')
            _write(((' accesskey="' + _tmp1) + '"'))
        u'view/maxlength'
        _tmp1 = _path(econtext['view'], econtext['request'], True, 'maxlength')
        if (_tmp1 is _default):
            _tmp1 = u''
        if ((_tmp1 is not None) and (_tmp1 is not False)):
            if (_tmp1.__class__ not in (str, unicode, int, float, )):
                _tmp1 = unicode(_translate(_tmp1, domain=_domain, mapping=None, target_language=target_language, default=None))
            else:
                if not isinstance(_tmp1, unicode):
                    _tmp1 = str(_tmp1)
            if ('&' in _tmp1):
                if (';' in _tmp1):
                    _tmp1 = _re_amp.sub('&amp;', _tmp1)
                else:
                    _tmp1 = _tmp1.replace('&', '&amp;')
            if ('<' in _tmp1):
                _tmp1 = _tmp1.replace('<', '&lt;')
            if ('>' in _tmp1):
                _tmp1 = _tmp1.replace('>', '&gt;')
            if ('"' in _tmp1):
                _tmp1 = _tmp1.replace('"', '&quot;')
            _write(((' maxlength="' + _tmp1) + '"'))
        _write(' />')
        return _out.getvalue()
    return render

__filename__ = 'd:\\svnxeebo\\packages\\trunk\\externals\\m01.grid\\src\\m01\\grid\\widget\\widget_input.pt'
registry[(None, True, '1488bdb950901f8f258549439ef6661a49aae984')] = bind()



###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: error.py 5564 2025-07-29 15:42:31Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.component

import m01.fs.exceptions

import z3c.form.error


@zope.component.adapter(m01.fs.exceptions.FileError, None,
        None, None, None, None)
class FileErrorViewSnippet(z3c.form.error.ErrorViewSnippet):
    """An error view for ValueError."""

    def createMessage(self):
        return self.error.args[0]


###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: exceptions.py 3048 2012-09-02 05:33:38Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.i18nmessageid
import zope.schema.interfaces

from pymongo.errors import PyMongoError

_ = zope.i18nmessageid.MessageFactory('p01')


class FileError(PyMongoError):
    __doc__ = _("""File upload exceptions""")


class CorruptFile(FileError):
    __doc__ = _("""Malformed file upload""")


class TooSmallFile(zope.schema.interfaces.TooSmall, FileError):
    __doc__ = _("""Too small file given""")


class TooBigFile(zope.schema.interfaces.TooBig, FileError):
    __doc__ = _("""Too big file given""")


class MissingFileNameExtension(zope.schema.interfaces.ValidationError):
    __doc__ = _("""Missing filename extension""")


class AllowedFormatError(zope.schema.interfaces.ValidationError):
    __doc__ = _("""Not allowed file format""")



###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id: base.py 4804 2018-03-16 04:27:22Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import m01.mongo.base
from m01.mongo.fieldproperty import MongoFieldProperty
from m01.mongo.fieldproperty import MongoBinaryProperty

from m01.fs import interfaces
import m01.fs.chunker


###############################################################################
#
# file and image shared core

class FileCore(object):
    """File base class"""

    data = MongoBinaryProperty(interfaces.IFileBaseSchema['data'])

    size = MongoFieldProperty(interfaces.IFileBaseSchema['size'])
    md5 = MongoFieldProperty(interfaces.IFileBaseSchema['md5'])
    filename = MongoFieldProperty(interfaces.IFileBaseSchema['filename'])
    contentType = MongoFieldProperty(interfaces.IFileBaseSchema['contentType'])
    encoding = MongoFieldProperty(interfaces.IFileBaseSchema['encoding'])
    uploadDate = MongoFieldProperty(interfaces.IFileBaseSchema['uploadDate'])
    encoding = MongoFieldProperty(interfaces.IFileBaseSchema['encoding'])

    removed = MongoFieldProperty(interfaces.IFileBaseSchema['removed'])

    def getFileWriter(self):
        """Returns a IChunkWriter"""
        raise NotImplementedError(
            "Subclass must implement getFileWriter method")

    def getFileReader(self):
        """Returns a IChunkReader"""
        raise NotImplementedError(
            "Subclass must implement getFileReader method")

    def applyFileUpload(self, fileUpload):
        """Apply FileUpload given from request publisher"""
        if not fileUpload or not fileUpload.filename:
            # empty string or None means no upload
            raise ValueError("Missing file upload data")
        elif self.removed:
            raise ValueError("Can't store data for removed files")
        writer = self.getFileWriter()
        writer.add(fileUpload)

    def __repr__(self):
        return "<%s %r>" % (self.__class__.__name__, self.__name__)


###############################################################################
#
# file

class FileBase(FileCore):
    """File base class"""

    def getFileWriter(self):
        """Returns a IChunkReader"""
        return m01.fs.chunker.FileChunkWriter(self)

    def getFileReader(self):
        """Returns a IChunkReader"""
        return m01.fs.chunker.FileChunkReader(self)


class FileItemBase(FileBase, m01.mongo.base.MongoItemBase):
    """Mongo file item base class."""

    _dumpNames = ['_id', '_pid', '_type', '_version', '__name__',
                  'created', 'modified', 'removed',
                  'data', 'size', 'md5', 'filename', 'contentType', 'encoding',
                  'uploadDate',
                  ]


class SecureFileItemBase(FileBase, m01.mongo.base.SecureMongoItemBase):
    """Secure mongo file item base class."""

    _dumpNames = ['_id', '_pid', '_type', '_version', '__name__',
                  'created', 'modified', 'removed',
                  '_ppmrow', '_ppmcol',
                  '_prmrow', '_prmcol',
                  '_rpmrow', '_rpmcol',
                  'data', 'size', 'md5', 'filename', 'contentType', 'encoding',
                  'uploadDate',
                  ]


###############################################################################
#
# image

class ImageBase(FileCore):
    """File base class"""

    width = MongoFieldProperty(interfaces.IImageSchema['width'])
    height = MongoFieldProperty(interfaces.IImageSchema['height'])

    def getFileWriter(self):
        """Returns a IChunkReader"""
        return m01.fs.chunker.ImageChunkWriter(self)

    def getFileReader(self):
        """Returns a IChunkReader"""
        return m01.fs.chunker.ImageChunkReader(self)


class ImageItemBase(ImageBase, m01.mongo.base.MongoItemBase):
    """Mongo image item base class."""

    _dumpNames = ['_id', '_pid', '_type', '_version', '__name__',
                  'created', 'modified', 'removed',
                  'data', 'size', 'md5', 'filename', 'contentType', 'encoding',
                  'uploadDate', 'width', 'height',
                  ]


class SecureImageItemBase(ImageBase, m01.mongo.base.SecureMongoItemBase):
    """Secure mongo image item base class."""

    _dumpNames = ['_id', '_pid', '_type', '_version', '__name__',
                  'created', 'modified', 'removed',
                  '_ppmrow', '_ppmcol',
                  '_prmrow', '_prmcol',
                  '_rpmrow', '_rpmcol',
                  'data', 'size', 'md5', 'filename', 'contentType', 'encoding',
                  'uploadDate', 'width', 'height',
                  ]
